/*
 * dlsof.h - Linux header file for lsof
 */


/*
 * Copyright 1994 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */


/*
 * $Id: dlsof.h,v 1.3 97/04/17 12:55:07 abe Exp $
 */


#if	!defined(LINUX_LSOF_H)
#define	LINUX_LSOF_H	1

#include <dirent.h>
#include <fcntl.h>
#include <malloc.h>
#include <mntent.h>
#include <stdlib.h>
#include <string.h>
#include <setjmp.h>
#include <unistd.h>

# if	LINUXV>=1147
#define	__KERNEL__	1
#undef	INT_MAX
#undef	UINT_MAX
#undef	LONG_MAX
#undef	ULONG_MAX
# endif	/* LINUXV>=1147 */

# if	LINUXV>=1999
#undef	d_namlen
# endif	/* LINUXV>=1999 */

#include <linux/socket.h>

# if	defined(HAS_ATOMIC_T) 
#include <asm/atomic.h>
#define	SOCK_RMEM_ALLOC	rmem_alloc.counter
#define	SOCK_WMEM_ALLOC	wmem_alloc.counter
# else	/* !defined(HAS_ATOMIC_T) */
#define	SOCK_RMEM_ALLOC	rmem_alloc
#define	SOCK_WMEM_ALLOC	wmem_alloc
# endif	/* defined(HAS_ATOMIC_T) */

# if	defined(HAS_SOCKET_SK)
#define	SOCK_SENT_SEQ	write_seq
#define	SOCKET_SOCK	sk
# else	/* !defined(HAS_SOCKET_SK) */
#define	SOCK_SENT_SEQ	sent_seq
#define	SOCKET_SOCK	data
# endif	/* defined(HAS_SOCKET_SK) */

#include <linux/net.h>
#include <linux/in.h>
#include <linux/fs.h>
#include <linux/tasks.h>
#include <linux/sched.h>
#include <linux/module.h>
#include <linux/unistd.h>

# if	defined(HAS_QUERY_MODULE)
#define	query_module(nm, wh, b, bl, r)	syscall(__NR_query_module, (nm), (wh), (b), (bl), (r))
# else	/* !defined(HAS_QUERY_MODULE) */
#define	get_kernel_syms(addr)	syscall(__NR_get_kernel_syms, (addr))
# endif	/* defined(HAS_QUERY_MODULE) */

# if	LINUXV<1147
#define	PWD	pwd
#define	ROOT	root
# else	/* LINUXV>=1147 */
#define	PWD	fs->pwd
#define	ROOT	fs->root
# endif	/* LINUXV<1147 */

#include <linux/mm.h>

# if	LINUXV<1147
#include <linux/ddi.h>
# endif	/* LINUXV<1147 */

# if	defined(HASNETDEVICE_H)
#include <linux/netdevice.h>
# endif	/* defined(HASNETDEVICE_H) */

# if	LINUXV<1300
#include <net/inet/sock.h>
# else	/* LINUXV>=1300 */
#include <net/sock.h>
# endif	/* LINUXV<1300 */

#include <sys/un.h>

# if	defined(HASUNMINSOCK) && HASUNMINSOCK==1
#include <net/af_unix.h>
# endif	/* defined(HASUNMINSOCK) && HASUNMINSOCK==1 */

# if	LINUXV<1300
#include <net/unix/unix.h>
# endif	/* LINUXV<1300 */

#include <netinet/protocols.h>
#define	IPPROTO_TCP	IP_TCP
#define	IPPROTO_UDP	IP_UDP
#define	IPPROTO_ICMP	IP_ICMP
#define	IPPROTO_IGMP	IP_IGMP
#define	IPPROTO_GGP	IP_GGP
#define	IPPROTO_EGP	IP_EGP
#define	IPPROTO_PUP	IP_PUP
#define	IPPROTO_IDP	IP_XNS_IDP
#define	IPPROTO_ND	IP_SUN_ND

#include <sys/sysmacros.h>


#define	COMP_P		const void
#define DEVINCR		1024	/* device table malloc() increment */
#define	FSNAMEL		4
#define	KA_T		off_t
#define	KMEM		"/dev/kmem"
#define MALLOC_P	void
#define FREE_P		MALLOC_P
#define MALLOC_S	size_t
#define QSORT_P		void
#define	READLEN_T	size_t
#define STRNCPY_L	size_t
#define	STRNML		32


/*
 * Global storage definitions (including their structure definitions)
 */

extern int Cfd;
extern struct file *Cfp;
extern int Kd;

struct nlist_private {
	char *n_name;			/* kernel symbol name */
	int n_type;			/* type: 0 = defined, 1 = undefined */
	unsigned long n_value;		/* address value */
};
#define	NLIST_TYPE	nlist_private
#define	NL_NAME		n_name

struct mounts {
        dev_t dev;              	/* st_dev */
        char *dir;              	/* directory */
        char *fsname;           	/* file system */
	ino_t inode;			/* st_ino */
	mode_t mode;			/* st_mode */
        struct mounts *next;    	/* forward link */
	dev_t rdev;			/* st_rdev */
};

struct sfile {
	char *aname;			/* argument file name */
	char *name;			/* file name (after readlink()) */
	char *devnm;			/* device name (optional) */
	dev_t dev;			/* device */
	umode_t mode;			/* S_IFMT mode bits from stat() */
	int type;			/* file type: 0 = file system
				 	 *	      1 = regular file */
	ino_t i;			/* inode number */
	int f;				/* file found flag */
	struct sfile *next;		/* forward link */
};

struct task_struct *Tsp;


/*
 * Definitions for rdev.c
 */

#define	DIRTYPE	dirent

# if	LINUXV<1208
#define HASDNAMLEN	1	/* struct DIRTYPE has d_namlen element */
# endif	/* LINUXV<1208 */


/*
 * Definition for rmnt.c
 */

#define MNTSKIP \
	{ if (strcmp(mp->mnt_type, MNTTYPE_IGNORE) == 0) \
		continue; }

#endif	/* LINUX_LSOF_H	*/
